
{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: replacing all controls with their text    }
{       representation                                  }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}


unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs,
  RVStyle, RVScroll, RichView, RVEdit, CRVData, RVItem, RVTable, RVUni,
  StdCtrls, ExtCtrls, RVTypes;

{$I RV_Defs.inc}

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    RVStyle2: TRVStyle;
    RichView1: TRichView;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ Returns a text representation for the Control }
function GetControlText(Control: TControl): TRVUnicodeString;
var
  s: String;
begin
  if Control is TEdit then
    s := TEdit(Control).Text
  else if Control is TComboBox then
  begin
    if TComboBox(Control).Style in [csSimple, csDropDown] then
      s := TComboBox(Control).Text
    else if TComboBox(Control).ItemIndex>=0 then
      s := TComboBox(Control).Items[TComboBox(Control).ItemIndex]
    else
      s := '';
  end
  else if Control is TRadioGroup then
  begin
    if TRadioGroup(Control).ItemIndex<0 then
      s := '(?)'
    else
      s := TRadioGroup(Control).Items[TRadioGroup(Control).ItemIndex];
    end
  else
    s := '';
  Result := TRVUnicodeString(s);
end;

{ Inserting text item in RVData at the ItemNo position }
{ some undocumented functions are used }
procedure InsertTextItem(RVData: TCustomRVData; ItemNo, StyleNo: Integer;
  Text: TRVUnicodeString; PageBreak, NewLine, NewPara: Boolean);
var
  TextItem: TRVTextItemInfo;
begin
  TextItem := RichViewTextItemClass.Create(RVData);
  TextItem.StyleNo := StyleNo;
  TextItem.SameAsPrev := not NewPara;
  if NewLine and not NewPara then
    TextItem.BR := True;
  if PageBreak then
    TextItem.PageBreakBefore := True;
  TextItem.Inserting(RVData, Text, False);
  RVData.Items.InsertObject(ItemNo, Text, TextItem);
  TextItem.Inserted(RVData, ItemNo);
end;

{ Changes all controls in RVData (and its sub-data) to their text
  representations }
procedure ControlsToText(RVData: TCustomRVData; var LastTextStyleNo: Integer);
var
  i,r,c: Integer;
  Control: TControl;
  ControlName: TRVUnicodeString;
  ControlText: TRVUnicodeString;
  ControlTag: TRVTag;
  ControlVAlign: TRVVAlign;
  table: TRVTableItemInfo;
  PageBreak, NewLine, NewPara: Boolean;
begin
  for i := 0 to RVData.ItemCount-1 do
    case RVData.GetItemStyle(i) of
      rvsTable:
        begin
          table := TRVTableItemInfo(RVData.GetItem(i));
          for r := 0 to table.RowCount-1 do
            for c := 0 to table.ColCount-1 do
              if table.Cells[r,c]<>nil then
                ControlsToText(table.Cells[r,c].GetRVData, LastTextStyleNo);

        end;
      rvsComponent:
        begin
          RVData.GetControlInfo(i, ControlName, Control, ControlVAlign, ControlTag);
          ControlText := GetControlText(Control);
          PageBreak := RVData.PageBreaksBeforeItems[i];
          NewLine := RVData.IsFromNewLine(i);
          NewPara := RVData.IsParaStart(i);
          RVData.DeleteItems(i, 1);
          InsertTextItem(RVData, i, LastTextStyleNo, ControlText,
            PageBreak, NewLine, NewPara);
        end;
      0..MaxInt: // text
        begin
          LastTextStyleNo := RVData.GetItemStyle(i);
        end;
    end;
end;

{ Creating a sample document }
procedure TForm1.FormCreate(Sender: TObject);
var rg: TRadioGroup;
    cmb: TComboBox;
begin
  { if you use older version of trichview, change rvvaAbsMiddle to rvvaBaseline }

  RichViewEdit1.Clear;

  RichViewEdit1.AddNL('If the weather is ', 0, 0);

  rg := TRadioGroup.Create(nil);
  RichViewEdit1.AddControl('', rg, -1, rvvaAbsMiddle);
  rg.Width := 80;
  rg.Height := 80;
  rg.ParentColor := False;
  rg.Color := clWhite;
  rg.Items.Text := 'sunny'#13'rainy'#13'cloudy';
  rg.ItemIndex := 0;

  RichViewEdit1.AddNL(' I will go ', 0, -1);

  cmb := TComboBox.Create(nil);
  RichViewEdit1.AddControl('', cmb, -1, rvvaAbsMiddle);
  cmb.Items.Text := 'shopping'#13'swimming'#13'for a walk';
  cmb.Style := csDropDownList;
  cmb.ItemIndex := 0;

  RichViewEdit1.AddNL('.', 0, -1);

  RichViewEdit1.Format;
  RegisterClasses([TEdit, TRadioGroup, TComboBox]);
end;

{ Replacing }
procedure TForm1.Button1Click(Sender: TObject);
var StyleNo: Integer;
    Stream: TMemoryStream;
begin
  // copying RichViewEdit1 -> RichView1
  Stream := TMemoryStream.Create;
  RichViewEdit1.SaveRVFToStream(Stream, False);
  Stream.Position := 0;
  RichView1.LoadRVFFromStream(Stream);
  Stream.Free;
  // replacing controls
  StyleNo := 0;
  ControlsToText(RichView1.RVData, StyleNo);
  {
  for optimization of the resulting document, I recommend to call
  NormalizeRichView(RichView1.RVData);
  here. This procedure can be found in RichViewActions, RVNormalize.pas
  }
  RichView1.Format;
end;

end.
